@@ -23,14 +23,18 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 
 import org.apache.logging.log4j.Level;
 import org.apache.logging.log4j.Marker;
 import org.apache.logging.log4j.ThreadContext;
 import org.apache.logging.log4j.core.LogEvent;
+import org.apache.logging.log4j.core.async.RingBufferLogEvent;
 import org.apache.logging.log4j.core.config.Property;
 import org.apache.logging.log4j.core.util.Clock;
 import org.apache.logging.log4j.core.util.ClockFactory;
+import org.apache.logging.log4j.core.util.DummyNanoClock;
+import org.apache.logging.log4j.core.util.NanoClock;
 import org.apache.logging.log4j.message.LoggerNameAwareMessage;
 import org.apache.logging.log4j.message.Message;
 import org.apache.logging.log4j.message.TimestampMessage;
@@ -42,22 +46,26 @@ import org.apache.logging.log4j.util.Strings;
 public class Log4jLogEvent implements LogEvent {
 
     private static final long serialVersionUID = -1351367343806656055L;
-    private static final Clock clock = ClockFactory.getClock();
+    private static final Clock CLOCK = ClockFactory.getClock();
+    private static volatile NanoClock nanoClock = new DummyNanoClock();
     private final String loggerFqcn;
     private final Marker marker;
     private final Level level;
     private final String loggerName;
     private final Message message;
     private final long timeMillis;
-    private transient final Throwable thrown;
+    private final transient Throwable thrown;
     private ThrowableProxy thrownProxy;
     private final Map<String, String> contextMap;
     private final ThreadContext.ContextStack contextStack;
-    private String threadName = null;
+    private String threadName;
     private StackTraceElement source;
     private boolean includeLocation;
     private boolean endOfBatch = false;
+    /** @since Log4J 2.4 */
+    private final transient long nanoTime;
 
+    /** LogEvent Builder helper class. */
     public static class Builder implements org.apache.logging.log4j.core.util.Builder<LogEvent> {
 
         private String loggerFqcn;
@@ -66,15 +74,50 @@ public class Log4jLogEvent implements LogEvent {
         private String loggerName;
         private Message message;
         private Throwable thrown;
-
-        public Builder setLoggerFqcn(final String loggerFqcn) {
-            this.loggerFqcn = loggerFqcn;
-            return this;
+        private long timeMillis = CLOCK.currentTimeMillis();
+        private ThrowableProxy thrownProxy;
+        private Map<String, String> contextMap = ThreadContext.getImmutableContext();
+        private ThreadContext.ContextStack contextStack = ThreadContext.getImmutableStack();
+        private String threadName = null;
+        private StackTraceElement source;
+        private boolean includeLocation;
+        private boolean endOfBatch = false;
+        private long nanoTime;
+        
+        public Builder() {
         }
-
-        public Builder setMarker(final Marker marker) {
-            this.marker = marker;
-            return this;
+        
+        public Builder(LogEvent other) {
+            Objects.requireNonNull(other);
+            if (other instanceof RingBufferLogEvent) {
+                RingBufferLogEvent evt = (RingBufferLogEvent) other;
+                evt.initializeBuilder(this);
+                return;
+            }
+            this.loggerFqcn = other.getLoggerFqcn();
+            this.marker = other.getMarker();
+            this.level = other.getLevel();
+            this.loggerName = other.getLoggerName();
+            this.message = other.getMessage();
+            this.timeMillis = other.getTimeMillis();
+            this.thrown = other.getThrown();
+            this.contextMap = other.getContextMap();
+            this.contextStack = other.getContextStack();
+            this.includeLocation = other.isIncludeLocation();
+            this.endOfBatch = other.isEndOfBatch();
+            this.nanoTime = other.getNanoTime();
+            
+            // Avoid unnecessarily initializing thrownProxy, threadName and source if possible
+            if (other instanceof Log4jLogEvent) {
+                Log4jLogEvent evt = (Log4jLogEvent) other;
+                this.thrownProxy = evt.thrownProxy;
+                this.source = evt.source;
+                this.threadName = evt.threadName;
+            } else {
+                this.thrownProxy = other.getThrownProxy();
+                this.source = other.getSource();
+                this.threadName = other.getThreadName();
+            }
         }
 
         public Builder setLevel(final Level level) {
@@ -82,11 +125,21 @@ public class Log4jLogEvent implements LogEvent {
             return this;
         }
 
+        public Builder setLoggerFqcn(final String loggerFqcn) {
+            this.loggerFqcn = loggerFqcn;
+            return this;
+        }
+
         public Builder setLoggerName(final String loggerName) {
             this.loggerName = loggerName;
             return this;
         }
 
+        public Builder setMarker(final Marker marker) {
+            this.marker = marker;
+            return this;
+        }
+
         public Builder setMessage(final Message message) {
             this.message = message;
             return this;
@@ -97,108 +150,181 @@ public class Log4jLogEvent implements LogEvent {
             return this;
         }
 
+        public Builder setTimeMillis(long timeMillis) {
+            this.timeMillis = timeMillis;
+            return this;
+        }
+
+        public Builder setThrownProxy(ThrowableProxy thrownProxy) {
+            this.thrownProxy = thrownProxy;
+            return this;
+        }
+
+        public Builder setContextMap(Map<String, String> contextMap) {
+            this.contextMap = contextMap;
+            return this;
+        }
+
+        public Builder setContextStack(ThreadContext.ContextStack contextStack) {
+            this.contextStack = contextStack;
+            return this;
+        }
+
+        public Builder setThreadName(String threadName) {
+            this.threadName = threadName;
+            return this;
+        }
+
+        public Builder setSource(StackTraceElement source) {
+            this.source = source;
+            return this;
+        }
+
+        public Builder setIncludeLocation(boolean includeLocation) {
+            this.includeLocation = includeLocation;
+            return this;
+        }
+
+        public Builder setEndOfBatch(boolean endOfBatch) {
+            this.endOfBatch = endOfBatch;
+            return this;
+        }
+
+        /**
+         * Sets the nano time for the event.
+         * @param nanoTime The value of the running Java Virtual Machine's high-resolution time source when the event
+         *          was created.
+         * @return this builder
+         */
+        public Builder setNanoTime(long nanoTime) {
+            this.nanoTime = nanoTime;
+            return this;
+        }
+
         @Override
         public Log4jLogEvent build() {
-            return new Log4jLogEvent(loggerName, marker, loggerFqcn, level, message, thrown);
+            final Log4jLogEvent result = new Log4jLogEvent(loggerName, marker, loggerFqcn, level, message, thrown,
+                    thrownProxy, contextMap, contextStack, threadName, source, timeMillis, nanoTime);
+            result.setIncludeLocation(includeLocation);
+            result.setEndOfBatch(endOfBatch);
+            return result;
         }
     }
 
+    /**
+     * Returns a new empty {@code Log4jLogEvent.Builder} with all fields empty.
+     * @return a new empty builder.
+     */
     public static Builder newBuilder() {
         return new Builder();
     }
 
     public Log4jLogEvent() {
-        this(clock.currentTimeMillis());
+        this(Strings.EMPTY, null, Strings.EMPTY, null, null, (Throwable) null, null, null, null, null, null,
+                CLOCK.currentTimeMillis(), nanoClock.nanoTime());
     }
 
     /**
-     *
-     */
-    public Log4jLogEvent(final long timestamp) {
-        this(Strings.EMPTY, null, Strings.EMPTY, null, null, (Throwable) null, null, null, null, null, timestamp);
-    }
+    *
+    * @deprecated use {@link Log4jLogEvent.Builder} instead. This constructor will be removed in an upcoming release.
+    */
+   @Deprecated
+   public Log4jLogEvent(final long timestamp) {
+       this(Strings.EMPTY, null, Strings.EMPTY, null, null, (Throwable) null, null, null, null, null, null,
+               timestamp, nanoClock.nanoTime());
+   }
 
-    /**
-     * Constructor.
-     * @param loggerName The name of the Logger.
-     * @param marker The Marker or null.
-     * @param loggerFQCN The fully qualified class name of the caller.
-     * @param level The logging Level.
-     * @param message The Message.
-     * @param t A Throwable or null.
-     */
-    public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
-                         final Message message, final Throwable t) {
-        this(loggerName, marker, loggerFQCN, level, message, null, t);
-    }
+   /**
+    * Constructor.
+    * @param loggerName The name of the Logger.
+    * @param marker The Marker or null.
+    * @param loggerFQCN The fully qualified class name of the caller.
+    * @param level The logging Level.
+    * @param message The Message.
+    * @param t A Throwable or null.
+    * @deprecated use {@link Log4jLogEvent.Builder} instead. This constructor will be removed in an upcoming release.
+    */
+   @Deprecated
+   public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
+                        final Message message, final Throwable t) {
+       this(loggerName, marker, loggerFQCN, level, message, null, t);
+   }
 
-    /**
-     * Constructor.
-     * @param loggerName The name of the Logger.
-     * @param marker The Marker or null.
-     * @param loggerFQCN The fully qualified class name of the caller.
-     * @param level The logging Level.
-     * @param message The Message.
-     * @param properties properties to add to the event.
-     * @param t A Throwable or null.
-     */
-    public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
-                         final Message message, final List<Property> properties, final Throwable t) {
-        this(loggerName, marker, loggerFQCN, level, message, t,
-            createMap(properties),
-            ThreadContext.getDepth() == 0 ? null : ThreadContext.cloneStack(), null,
-            null,
-            // LOG4J2-628 use log4j.Clock for timestamps
-            // LOG4J2-744 unless TimestampMessage already has one
-            message instanceof TimestampMessage ? ((TimestampMessage) message).getTimestamp() :
-                clock.currentTimeMillis());
-    }
+   /**
+    * Constructor.
+    * @param loggerName The name of the Logger.
+    * @param marker The Marker or null.
+    * @param loggerFQCN The fully qualified class name of the caller.
+    * @param level The logging Level.
+    * @param message The Message.
+    * @param properties properties to add to the event.
+    * @param t A Throwable or null.
+    */
+   // This constructor is called from LogEventFactories.
+   public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
+                        final Message message, final List<Property> properties, final Throwable t) {
+       this(loggerName, marker, loggerFQCN, level, message, t, null,
+           createMap(properties),
+           ThreadContext.getDepth() == 0 ? null : ThreadContext.cloneStack(), // mutable copy
+           null, // thread name
+           null, // stack trace element
+           // LOG4J2-628 use log4j.Clock for timestamps
+           // LOG4J2-744 unless TimestampMessage already has one
+           message instanceof TimestampMessage ? ((TimestampMessage) message).getTimestamp() :
+               CLOCK.currentTimeMillis(),
+           nanoClock.nanoTime());
+   }
 
-    /**
-     * Constructor.
-     * @param loggerName The name of the Logger.
-     * @param marker The Marker or null.
-     * @param loggerFQCN The fully qualified class name of the caller.
-     * @param level The logging Level.
-     * @param message The Message.
-     * @param t A Throwable or null.
-     * @param mdc The mapped diagnostic context.
-     * @param ndc the nested diagnostic context.
-     * @param threadName The name of the thread.
-     * @param location The locations of the caller.
-     * @param timestamp The timestamp of the event.
-     */
-    public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
-                         final Message message, final Throwable t, final Map<String, String> mdc,
-                         final ThreadContext.ContextStack ndc, final String threadName,
-                         final StackTraceElement location, final long timestamp) {
-        this(loggerName, marker, loggerFQCN, level, message, t, null, mdc, ndc, threadName,
-                location, timestamp);
-    }
+   /**
+    * Constructor.
+    * @param loggerName The name of the Logger.
+    * @param marker The Marker or null.
+    * @param loggerFQCN The fully qualified class name of the caller.
+    * @param level The logging Level.
+    * @param message The Message.
+    * @param t A Throwable or null.
+    * @param mdc The mapped diagnostic context.
+    * @param ndc the nested diagnostic context.
+    * @param threadName The name of the thread.
+    * @param location The locations of the caller.
+    * @param timestampMillis The timestamp of the event.
+    * @deprecated use {@link Log4jLogEvent.Builder} instead. This constructor will be removed in an upcoming release.
+    */
+   @Deprecated
+public Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
+                        final Message message, final Throwable t, final Map<String, String> mdc,
+                        final ThreadContext.ContextStack ndc, final String threadName,
+                        final StackTraceElement location, final long timestampMillis) {
+       this(loggerName, marker, loggerFQCN, level, message, t, null, mdc, ndc, threadName,
+               location, timestampMillis, nanoClock.nanoTime());
+   }
 
-    /**
-     * Create a new LogEvent.
-     * @param loggerName The name of the Logger.
-     * @param marker The Marker or null.
-     * @param loggerFQCN The fully qualified class name of the caller.
-     * @param level The logging Level.
-     * @param message The Message.
-     * @param thrown A Throwable or null.
-     * @param thrownProxy A ThrowableProxy or null.
-     * @param mdc The mapped diagnostic context.
-     * @param ndc the nested diagnostic context.
-     * @param threadName The name of the thread.
-     * @param location The locations of the caller.
-     * @param timestamp The timestamp of the event.
-     */
+   /**
+    * Create a new LogEvent.
+    * @param loggerName The name of the Logger.
+    * @param marker The Marker or null.
+    * @param loggerFQCN The fully qualified class name of the caller.
+    * @param level The logging Level.
+    * @param message The Message.
+    * @param thrown A Throwable or null.
+    * @param thrownProxy A ThrowableProxy or null.
+    * @param mdc The mapped diagnostic context.
+    * @param ndc the nested diagnostic context.
+    * @param threadName The name of the thread.
+    * @param location The locations of the caller.
+    * @param timestamp The timestamp of the event.
+    * @return a new LogEvent
+    * @deprecated use {@link Log4jLogEvent.Builder} instead. This method will be removed in an upcoming release.
+    */
+    @Deprecated
     public static Log4jLogEvent createEvent(final String loggerName, final Marker marker, final String loggerFQCN,
-                                            final Level level, final Message message, final Throwable thrown, 
+                                            final Level level, final Message message, final Throwable thrown,
                                             final ThrowableProxy thrownProxy,
                                             final Map<String, String> mdc, final ThreadContext.ContextStack ndc,
                                             final String threadName, final StackTraceElement location,
                                             final long timestamp) {
         final Log4jLogEvent result = new Log4jLogEvent(loggerName, marker, loggerFQCN, level, message, thrown, 
-                thrownProxy, mdc, ndc, threadName, location, timestamp);
+                thrownProxy, mdc, ndc, threadName, location, timestamp, nanoClock.nanoTime());
         return result;
     }
 
@@ -216,11 +342,13 @@ public class Log4jLogEvent implements LogEvent {
      * @param threadName The name of the thread.
      * @param source The locations of the caller.
      * @param timestamp The timestamp of the event.
+     * @param nanoTime The value of the running Java Virtual Machine's high-resolution time source when the event was
+     *          created.
      */
     private Log4jLogEvent(final String loggerName, final Marker marker, final String loggerFQCN, final Level level,
-            final Message message, final Throwable thrown, final ThrowableProxy thrownProxy, 
-            final Map<String, String> contextMap, final ThreadContext.ContextStack contextStack, 
-            final String threadName, final StackTraceElement source, final long timestamp) {
+            final Message message, final Throwable thrown, final ThrowableProxy thrownProxy,
+            final Map<String, String> contextMap, final ThreadContext.ContextStack contextStack,
+            final String threadName, final StackTraceElement source, final long timestampMillis, final long nanoTime) {
         this.loggerName = loggerName;
         this.marker = marker;
         this.loggerFqcn = loggerFQCN;
@@ -230,23 +358,23 @@ public class Log4jLogEvent implements LogEvent {
         this.thrownProxy = thrownProxy;
         this.contextMap = contextMap == null ? ThreadContext.EMPTY_MAP : contextMap;
         this.contextStack = contextStack == null ? ThreadContext.EMPTY_STACK : contextStack;
-        this.timeMillis = message instanceof TimestampMessage ? ((TimestampMessage) message).getTimestamp() : timestamp;
+        this.timeMillis = message instanceof TimestampMessage
+                ? ((TimestampMessage) message).getTimestamp()
+                : timestampMillis;
         this.threadName = threadName;
         this.source = source;
         if (message != null && message instanceof LoggerNameAwareMessage) {
             ((LoggerNameAwareMessage) message).setLoggerName(loggerName);
         }
+        this.nanoTime = nanoTime;
     }
 
     private static Map<String, String> createMap(final List<Property> properties) {
         final Map<String, String> contextMap = ThreadContext.getImmutableContext();
-        if (contextMap == null && (properties == null || properties.isEmpty())) {
-            return null;
-        }
         if (properties == null || properties.isEmpty()) {
-            return contextMap; // contextMap is not null
+            return contextMap; // may be ThreadContext.EMPTY_MAP but not null
         }
-        final Map<String, String> map = new HashMap<String, String>(contextMap);
+        final Map<String, String> map = new HashMap<>(contextMap);
 
         for (final Property prop : properties) {
             if (!map.containsKey(prop.getName())) {
@@ -255,6 +383,34 @@ public class Log4jLogEvent implements LogEvent {
         }
         return Collections.unmodifiableMap(map);
     }
+    
+    /**
+     * Returns the {@code NanoClock} to use for creating the nanoTime timestamp of log events.
+     * @return the {@code NanoClock} to use for creating the nanoTime timestamp of log events
+     */
+    public static NanoClock getNanoClock() {
+        return nanoClock;
+    }
+    
+    /**
+     * Sets the {@code NanoClock} to use for creating the nanoTime timestamp of log events.
+     * <p>
+     * FOR INTERNAL USE. This method may be called with a different {@code NanoClock} implementation when the
+     * configuration changes.
+     * 
+     * @param nanoClock the {@code NanoClock} to use for creating the nanoTime timestamp of log events
+     */
+    public static void setNanoClock(NanoClock nanoClock) {
+        Log4jLogEvent.nanoClock = Objects.requireNonNull(nanoClock, "NanoClock must be non-null");
+    }
+    
+    /**
+     * Returns a new fully initialized {@code Log4jLogEvent.Builder} containing a copy of all fields of this event.
+     * @return a new fully initialized builder.
+     */
+    public Builder asBuilder() {
+        return new Builder(this);
+    }
 
     /**
      * Returns the logging Level.
@@ -415,6 +571,11 @@ public class Log4jLogEvent implements LogEvent {
         this.endOfBatch = endOfBatch;
     }
 
+    @Override
+    public long getNanoTime() {
+        return nanoTime;
+    }
+
     /**
      * Creates a LogEventProxy that can be serialized.
      * @return a LogEventProxy.
@@ -435,15 +596,13 @@ public class Log4jLogEvent implements LogEvent {
     }
 
     public static Log4jLogEvent deserialize(final Serializable event) {
-        if (event == null) {
-            throw new NullPointerException("Event cannot be null");
-        }
+        Objects.requireNonNull(event, "Event cannot be null");
         if (event instanceof LogEventProxy) {
             final LogEventProxy proxy = (LogEventProxy) event;
             final Log4jLogEvent result = new Log4jLogEvent(proxy.loggerName, proxy.marker,
                     proxy.loggerFQCN, proxy.level, proxy.message,
                     proxy.thrown, proxy.thrownProxy, proxy.contextMap, proxy.contextStack, proxy.threadName,
-                    proxy.source, proxy.timeMillis);
+                    proxy.source, proxy.timeMillis, proxy.nanoTime);
             result.setEndOfBatch(proxy.isEndOfBatch);
             result.setIncludeLocation(proxy.isLocationRequired);
             return result;
@@ -485,6 +644,9 @@ public class Log4jLogEvent implements LogEvent {
         if (timeMillis != that.timeMillis) {
             return false;
         }
+        if (nanoTime != that.nanoTime) {
+            return false;
+        }
         if (loggerFqcn != null ? !loggerFqcn.equals(that.loggerFqcn) : that.loggerFqcn != null) {
             return false;
         }
@@ -524,12 +686,14 @@ public class Log4jLogEvent implements LogEvent {
 
     @Override
     public int hashCode() {
+        // Check:OFF: MagicNumber
         int result = loggerFqcn != null ? loggerFqcn.hashCode() : 0;
         result = 31 * result + (marker != null ? marker.hashCode() : 0);
         result = 31 * result + (level != null ? level.hashCode() : 0);
         result = 31 * result + loggerName.hashCode();
         result = 31 * result + message.hashCode();
         result = 31 * result + (int) (timeMillis ^ (timeMillis >>> 32));
+        result = 31 * result + (int) (nanoTime ^ (nanoTime >>> 32));
         result = 31 * result + (thrown != null ? thrown.hashCode() : 0);
         result = 31 * result + (thrownProxy != null ? thrownProxy.hashCode() : 0);
         result = 31 * result + (contextMap != null ? contextMap.hashCode() : 0);
@@ -538,6 +702,7 @@ public class Log4jLogEvent implements LogEvent {
         result = 31 * result + (source != null ? source.hashCode() : 0);
         result = 31 * result + (includeLocation ? 1 : 0);
         result = 31 * result + (endOfBatch ? 1 : 0);
+        // Check:ON: MagicNumber
         return result;
     }
 
@@ -561,6 +726,8 @@ public class Log4jLogEvent implements LogEvent {
         private final StackTraceElement source;
         private final boolean isLocationRequired;
         private final boolean isEndOfBatch;
+        /** @since Log4J 2.4 */
+        private final transient long nanoTime;
 
         public LogEventProxy(final Log4jLogEvent event, final boolean includeLocation) {
             this.loggerFQCN = event.loggerFqcn;
@@ -577,6 +744,7 @@ public class Log4jLogEvent implements LogEvent {
             this.threadName = event.getThreadName();
             this.isLocationRequired = includeLocation;
             this.isEndOfBatch = event.endOfBatch;
+            this.nanoTime = event.nanoTime;
         }
 
         /**
@@ -585,7 +753,7 @@ public class Log4jLogEvent implements LogEvent {
          */
         protected Object readResolve() {
             final Log4jLogEvent result = new Log4jLogEvent(loggerName, marker, loggerFQCN, level, message, thrown,
-                    thrownProxy, contextMap, contextStack, threadName, source, timeMillis);
+                    thrownProxy, contextMap, contextStack, threadName, source, timeMillis, nanoTime);
             result.setEndOfBatch(isEndOfBatch);
             result.setIncludeLocation(isLocationRequired);
             return result;
