@@ -17,6 +17,7 @@
 package org.apache.logging.log4j.core.layout;
 
 import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.GregorianCalendar;
@@ -47,12 +48,12 @@ import org.apache.logging.log4j.core.pattern.PatternConverter;
 import org.apache.logging.log4j.core.pattern.PatternFormatter;
 import org.apache.logging.log4j.core.pattern.PatternParser;
 import org.apache.logging.log4j.core.pattern.ThrowablePatternConverter;
-import org.apache.logging.log4j.core.util.Charsets;
 import org.apache.logging.log4j.core.util.NetUtils;
 import org.apache.logging.log4j.core.util.Patterns;
 import org.apache.logging.log4j.message.Message;
 import org.apache.logging.log4j.message.StructuredDataId;
 import org.apache.logging.log4j.message.StructuredDataMessage;
+import org.apache.logging.log4j.util.StringBuilders;
 import org.apache.logging.log4j.util.Strings;
 
 
@@ -148,7 +149,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             final String[] array = excludes.split(Patterns.COMMA_SEPARATOR);
             if (array.length > 0) {
                 c = new ExcludeChecker();
-                mdcExcludes = new ArrayList<String>(array.length);
+                mdcExcludes = new ArrayList<>(array.length);
                 for (final String str : array) {
                     mdcExcludes.add(str.trim());
                 }
@@ -162,7 +163,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             final String[] array = includes.split(Patterns.COMMA_SEPARATOR);
             if (array.length > 0) {
                 c = new IncludeChecker();
-                mdcIncludes = new ArrayList<String>(array.length);
+                mdcIncludes = new ArrayList<>(array.length);
                 for (final String str : array) {
                     mdcIncludes.add(str.trim());
                 }
@@ -175,7 +176,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         if (required != null) {
             final String[] array = required.split(Patterns.COMMA_SEPARATOR);
             if (array.length > 0) {
-                mdcRequired = new ArrayList<String>(array.length);
+                mdcRequired = new ArrayList<>(array.length);
                 for (final String str : array) {
                     mdcRequired.add(str.trim());
                 }
@@ -188,18 +189,18 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
         this.checker = c != null ? c : noopChecker;
         final String name = config == null ? null : config.getName();
-        configName = name != null && name.length() > 0 ? name : null;
+        configName = Strings.isNotEmpty(name) ? name : null;
         this.fieldFormatters = createFieldFormatters(loggerFields, config);
     }
 
     private Map<String, FieldFormatter> createFieldFormatters(final LoggerFields[] loggerFields,
             final Configuration config) {
-        final Map<String, FieldFormatter> sdIdMap = new HashMap<String, FieldFormatter>();
+        final Map<String, FieldFormatter> sdIdMap = new HashMap<>();
 
         if (loggerFields != null) {
             for (final LoggerFields lField : loggerFields) {
                 final StructuredDataId key = lField.getSdId() == null ? mdcSdId : lField.getSdId();
-                final Map<String, List<PatternFormatter>> sdParams = new HashMap<String, List<PatternFormatter>>();
+                final Map<String, List<PatternFormatter>> sdParams = new HashMap<>();
                 final Map<String, String> fields = lField.getMap();
                 if (!fields.isEmpty()) {
                     final PatternParser fieldParser = createPatternParser(config, null);
@@ -249,7 +250,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
      */
     @Override
     public Map<String, String> getContentFormat() {
-        final Map<String, String> result = new HashMap<String, String>();
+        final Map<String, String> result = new HashMap<>();
         result.put("structured", "true");
         result.put("formatType", "RFC5424");
         return result;
@@ -357,7 +358,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             return;
         }
 
-        final Map<String, StructuredDataElement> sdElements = new HashMap<String, StructuredDataElement>();
+        final Map<String, StructuredDataElement> sdElements = new HashMap<>();
         final Map<String, String> contextMap = event.getContextMap();
 
         if (mdcRequired != null) {
@@ -373,13 +374,14 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
 
         if (includeMdc && contextMap.size() > 0) {
-            if (sdElements.containsKey(mdcSdId.toString())) {
-                final StructuredDataElement union = sdElements.get(mdcSdId.toString());
+            final String mdcSdIdStr = mdcSdId.toString();
+            final StructuredDataElement union = sdElements.get(mdcSdIdStr);
+            if (union != null) {
                 union.union(contextMap);
-                sdElements.put(mdcSdId.toString(), union);
+                sdElements.put(mdcSdIdStr, union);
             } else {
                 final StructuredDataElement formattedContextMap = new StructuredDataElement(contextMap, false);
-                sdElements.put(mdcSdId.toString(), formattedContextMap);
+                sdElements.put(mdcSdIdStr, formattedContextMap);
             }
         }
 
@@ -532,16 +534,17 @@ public final class Rfc5424Layout extends AbstractStringLayout {
     }
 
     private void appendMap(final String prefix, final Map<String, String> map, final StringBuilder sb,
-                           final ListChecker checker) {
-        final SortedMap<String, String> sorted = new TreeMap<String, String>(map);
+            final ListChecker checker) {
+        final SortedMap<String, String> sorted = new TreeMap<>(map);
         for (final Map.Entry<String, String> entry : sorted.entrySet()) {
             if (checker.check(entry.getKey()) && entry.getValue() != null) {
                 sb.append(' ');
                 if (prefix != null) {
                     sb.append(prefix);
                 }
-                sb.append(escapeNewlines(escapeSDParams(entry.getKey()), escapeNewLine)).append("=\"")
-                    .append(escapeNewlines(escapeSDParams(entry.getValue()), escapeNewLine)).append('"');
+                final String safeKey = escapeNewlines(escapeSDParams(entry.getKey()), escapeNewLine);
+                final String safeValue = escapeNewlines(escapeSDParams(entry.getValue()), escapeNewLine);
+                StringBuilders.appendKeyDqValue(sb, safeKey, safeValue);
             }
         }
     }
@@ -644,14 +647,13 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             @PluginAttribute(value = "useTlsMessageFormat", defaultBoolean = false) final boolean useTlsMessageFormat, // RFC 5425
             @PluginElement("LoggerFields") final LoggerFields[] loggerFields,
             @PluginConfiguration final Configuration config) {
-        final Charset charset = Charsets.UTF_8;
         if (includes != null && excludes != null) {
             LOGGER.error("mdcIncludes and mdcExcludes are mutually exclusive. Includes wil be ignored");
             includes = null;
         }
 
         return new Rfc5424Layout(config, facility, id, enterpriseNumber, includeMDC, newLine, escapeNL, mdcId, mdcPrefix,
-                eventPrefix, appName, msgId, excludes, includes, required, charset, exceptionPattern,
+                eventPrefix, appName, msgId, excludes, includes, required, StandardCharsets.UTF_8, exceptionPattern,
                 useTlsMessageFormat, loggerFields);
     }
 
@@ -666,7 +668,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
 
         public StructuredDataElement format(final LogEvent event) {
-            final Map<String, String> map = new HashMap<String, String>();
+            final Map<String, String> map = new HashMap<>();
 
             for (final Map.Entry<String, List<PatternFormatter>> entry : delegateMap.entrySet()) {
                 final StringBuilder buffer = new StringBuilder();
