@@ -90,8 +90,6 @@ public class FlumePersistentManager extends FlumeAvroManager {
 
     private final SecretKey secretKey;
 
-    private final int delayMillis;
-
     private final int lockTimeoutRetryCount;
 
     private final ExecutorService threadPool;
@@ -118,8 +116,7 @@ public class FlumePersistentManager extends FlumeAvroManager {
                                      final int requestTimeout, final int delay, final Database database,
                                      final Environment environment, final SecretKey secretKey,
                                      final int lockTimeoutRetryCount) {
-        super(name, shortName, agents, batchSize, retries, connectionTimeout, requestTimeout);
-        this.delayMillis = delay;
+        super(name, shortName, agents, batchSize, delay, retries, connectionTimeout, requestTimeout);
         this.database = database;
         this.environment = environment;
         dbCount.set(database.count());
@@ -202,15 +199,15 @@ public class FlumePersistentManager extends FlumeAvroManager {
             final Future<Integer> future = threadPool.submit(new BDBWriter(keyData, eventData, environment, database,
                 gate, dbCount, getBatchSize(), lockTimeoutRetryCount));
             boolean interrupted = false;
-            int count = 0;
+            int ieCount = 0;
             do {
                 try {
                     future.get();
                 } catch (final InterruptedException ie) {
                     interrupted = true;
-                    ++count;
+                    ++ieCount;
                 }
-            } while (interrupted && count <= 1);
+            } while (interrupted && ieCount <= 1);
 
         } catch (final Exception ex) {
             throw new LoggingException("Exception occurred writing log event", ex);
@@ -395,7 +392,7 @@ public class FlumePersistentManager extends FlumeAvroManager {
             Database database = null;
             Environment environment = null;
 
-            final Map<String, String> properties = new HashMap<String, String>();
+            final Map<String, String> properties = new HashMap<>();
             if (data.properties != null) {
                 for (final Property property : data.properties) {
                     properties.put(property.getName(), property.getValue());
@@ -515,14 +512,14 @@ public class FlumePersistentManager extends FlumeAvroManager {
 
         @Override
         public void run() {
-            LOGGER.trace("WriterThread started - batch size = " + batchSize + ", delayMillis = " + manager.delayMillis);
-            long nextBatchMillis = System.currentTimeMillis() + manager.delayMillis;
+            LOGGER.trace("WriterThread started - batch size = " + batchSize + ", delayMillis = " + manager.getDelayMillis());
+            long nextBatchMillis = System.currentTimeMillis() + manager.getDelayMillis();
             while (!shutdown) {
                 final long nowMillis = System.currentTimeMillis();
                 final long dbCount = database.count();
                 dbCounter.set(dbCount);
                 if (dbCount >= batchSize || dbCount > 0 && nextBatchMillis <= nowMillis) {
-                    nextBatchMillis = nowMillis + manager.delayMillis;
+                    nextBatchMillis = nowMillis + manager.getDelayMillis();
                     try {
                         boolean errors = false;
                         final DatabaseEntry key = new DatabaseEntry();
@@ -621,7 +618,7 @@ public class FlumePersistentManager extends FlumeAvroManager {
                             }
                         }
                         if (errors) {
-                            Thread.sleep(manager.delayMillis);
+                            Thread.sleep(manager.getDelayMillis());
                             continue;
                         }
                     } catch (final Exception ex) {
@@ -629,7 +626,7 @@ public class FlumePersistentManager extends FlumeAvroManager {
                     }
                 } else {
                     if (nextBatchMillis <= nowMillis) {
-                        nextBatchMillis = nowMillis + manager.delayMillis;
+                        nextBatchMillis = nowMillis + manager.getDelayMillis();
                     }
                     try {
                         final long interval = nextBatchMillis - nowMillis;
@@ -810,7 +807,7 @@ public class FlumePersistentManager extends FlumeAvroManager {
                 dais.read(bytes, 0, length);
                 event.setBody(bytes);
                 length = dais.readInt();
-                final Map<String, String> map = new HashMap<String, String>(length);
+                final Map<String, String> map = new HashMap<>(length);
                 for (int i = 0; i < length; ++i) {
                     final String headerKey = dais.readUTF();
                     final String value = dais.readUTF();
